@@ -1,3 +1,17 @@
+function pairsByKeys (t, f)
+    local a = {}
+    for n in pairs(t) do table.insert(a, n) end
+    table.sort(a, f)
+    local i = 0
+    local iter = function ()
+        i = i + 1
+        if a[i] == nil then return nil
+        else return a[i], t[a[i]]
+        end
+    end
+    return iter
+end
+
 function pick_random(t)
     if t and #t>0 then
         return t[trandom(#t)+1]
@@ -17,7 +31,8 @@ end
 function pick_random_conditional(t,cond,...)
     local is_fun=type(cond)=="function"
     local candidates={}
-    for k,v in ipairs(t) do
+    -- we use pairsByKeys so the RNG seed is stable
+    for k,v in pairsByKeys(t) do
         if is_fun then 
             if cond(k,v,...) then
                 candidates[#candidates+1]=v
@@ -32,7 +47,8 @@ end
 function pick_random_conditional_pairs(t,cond,...)
     local is_fun=type(cond)=="function"
     local candidates={}
-    for k,v in pairs(t) do
+    -- we use pairsByKeys so the RNG seed is stable
+    for k,v in pairsByKeys(t) do
         if is_fun then 
             if cond(k,v,...) then
                 candidates[#candidates+1]=k
@@ -101,7 +117,18 @@ end
 function pick_random_pairs(tbl)
     if not tbl then return nil end
     local cand={}
-    for k,v in pairs(tbl) do
+    -- we use pairsByKeys so the RNG seed is stable
+    for k,v in pairsByKeys(tbl) do
+        if v then cand[#cand+1]=k end
+    end
+    return pick_random(cand)
+end
+
+function pick_random_pairs_with_sort(tbl,f)
+    if not tbl then return nil end
+    local cand={}
+    -- we use pairsByKeys so the RNG seed is stable
+    for k,v in pairsByKeys(tbl,f) do
         if v then cand[#cand+1]=k end
     end
     return pick_random(cand)
@@ -116,6 +143,7 @@ function log(...)
     lua_log(s)
 end
 
+--------------------------
 debug_level=0
 
 function get_caller_loc_string()
@@ -142,6 +170,7 @@ function partial_function(f,arg)
 end
 
 function pick_weighted_from_table(tbl)
+    -- we assume this table is sorted for RNG stability before the call since pairsByKeys doesn't sort arrays
     local l=get_debug_logger(4)
     local o=get_debug_logger(-10)
     l("Picking weighted table at ",get_caller_loc_string)
@@ -230,7 +259,8 @@ function generate_from_list(tbl,...)
     local results={}
     local total_weight=0
     local o=get_debug_logger(-10)
-    for k,v in pairs(tbl) do
+    -- we use pairsByKeys so the RNG seed is stable
+    for k,v in pairsByKeys(tbl) do
         local result = v(...)
         if result then
             if not result.weight then
