@@ -185,25 +185,27 @@ interactions.secrets.necromancer=function(idx,sph)
         [CDI:DEFAULT_ICON:ADVENTURE_INTERACTION_ICON_RAISE_INTELLIGENT_UNDEAD]
     ]])
     end
-    if bogeyman then
-        tbl=split_to_lines(tbl,[[
-        [CE_CAN_DO_INTERACTION:START:0:ABRUPT]
-        [CDI:ADV_NAME:Summon bogeymen]
-        [CDI:INTERACTION:]]..sum_b_token..[[]
-        [CDI:VERB:call upon the night:calls upon the night:NA]
-        [CDI:WAIT_PERIOD:100]
-        [CDI:DEFAULT_ICON:ADVENTURE_INTERACTION_ICON_SUMMON_BOGEYMAN]
-    ]])
-    end
-    if nightmare then
-        tbl=split_to_lines(tbl,[[
-        [CE_CAN_DO_INTERACTION:START:0:ABRUPT]
-        [CDI:ADV_NAME:Summon nightmare]
-        [CDI:INTERACTION:]]..sum_n_token..[[]
-        [CDI:VERB:call upon the night:calls upon the night:NA]
-        [CDI:WAIT_PERIOD:12000]
-        [CDI:DEFAULT_ICON:ADVENTURE_INTERACTION_ICON_SUMMON_BOGEYMAN]
-    ]])
+    if summon then 
+        if bogeyman then
+            tbl=split_to_lines(tbl,[[
+            [CE_CAN_DO_INTERACTION:START:0:ABRUPT]
+            [CDI:ADV_NAME:Summon bogeymen]
+            [CDI:INTERACTION:]]..sum_b_token..[[]
+            [CDI:VERB:call upon the night:calls upon the night:NA]
+            [CDI:WAIT_PERIOD:100]
+            [CDI:DEFAULT_ICON:ADVENTURE_INTERACTION_ICON_SUMMON_BOGEYMAN]
+        ]])
+        end
+        if nightmare then
+            tbl=split_to_lines(tbl,[[
+            [CE_CAN_DO_INTERACTION:START:0:ABRUPT]
+            [CDI:ADV_NAME:Summon nightmare]
+            [CDI:INTERACTION:]]..sum_n_token..[[]
+            [CDI:VERB:call upon the night:calls upon the night:NA]
+            [CDI:WAIT_PERIOD:12000]
+            [CDI:DEFAULT_ICON:ADVENTURE_INTERACTION_ICON_SUMMON_BOGEYMAN]
+        ]])
+        end
     end
     if ghoul then
         tbl=split_to_lines(tbl,[[
